@@ -36,9 +36,9 @@
 #define __STDIO_THREADSAFE
 #endif
 
-#ifdef __UCLIBC_HAVE_LFS__
+#ifdef __UCLIBC_HAS_LFS__
 #define __STDIO_LARGE_FILES
-#endif /* __UCLIBC_HAVE_LFS__ */
+#endif /* __UCLIBC_HAS_LFS__ */
 
 #ifdef __UCLIBC_HAS_WCHAR__
 #define __STDIO_WIDE
@@ -47,22 +47,22 @@
 /* Make sure defines related to large files are consistent. */
 #ifdef _LIBC
 
-#ifdef __UCLIBC_HAVE_LFS__
+#ifdef __UCLIBC_HAS_LFS__
 #undef __USE_LARGEFILE
 #undef __USE_LARGEFILE64
 #undef __USE_FILE_OFFSET64
 /* if we're actually building uClibc with large file support, only define... */
 #define __USE_LARGEFILE64	1
-#endif /* __UCLIBC_HAVE_LFS__ */
+#endif /* __UCLIBC_HAS_LFS__ */
 
 #else  /* not _LIBC */
 
-#ifndef __UCLIBC_HAVE_LFS__
+#ifndef __UCLIBC_HAS_LFS__
 #if defined(__LARGEFILE64_SOURCE) || defined(__USE_LARGEFILE64) \
     || defined(__USE_FILE_OFFSET64)
 #error Sorry... uClibc was built without large file support!
 #endif
-#endif /* __UCLIBC_HAVE_LFS__ */
+#endif /* __UCLIBC_HAS_LFS__ */
 
 #endif /* _LIBC */
 
@@ -463,6 +463,7 @@ extern void __stdio_validate_FILE(FILE *stream);
 
 typedef enum {
 	__UIM_DECIMAL = 0,
+	__UIM_GROUP = ',',			/* Base 10 locale-dependent grouping. */
 	__UIM_LOWER = 'a' - 10,
 	__UIM_UPPER = 'A' - 10,
 } __UIM_CASE;
